@@ -53,50 +53,52 @@
 #define FCLASS_SNAN          (1 << 8)
 #define FCLASS_QNAN          (1 << 9)
 
-
 #define FCLASS_INF           (FCLASS_NEG_INF | FCLASS_POS_INF)
 #define FCLASS_ZERO          (FCLASS_NEG_ZERO | FCLASS_POS_ZERO)
 #define FCLASS_NORMAL        (FCLASS_NEG_NORMAL | FCLASS_POS_NORMAL)
 #define FCLASS_SUBNORMAL     (FCLASS_NEG_SUBNORMAL | FCLASS_POS_SUBNORMAL)
 #define FCLASS_NAN           (FCLASS_SNAN | FCLASS_QNAN)
-#endif
-
-/**
- * Not availabe for all compilers.
- * In case of absence, fall back to normal function calls
- */
-#ifdef __GNUC_GNU_INLINE__
-# define __declare_riscv_macro(type) extern __inline type __attribute((gnu_inline, always_inline))
-# define __declare_riscv_macro_fclass(type) extern __inline type __attribute((gnu_inline, always_inline))
-#else
-# define __declare_riscv_macro_fclass(type) static __inline type
-#endif
-
-/* We always need the fclass functions */
 
 #if __riscv_flen >= 64
-__declare_riscv_macro_fclass(long)
-_fclass_d(double x)
-{
-	long fclass;
-	__asm __volatile ("fclass.d\t%0, %1" : "=r" (fclass) : "f" (x));
-	return fclass;
-}
+
+/* anything with a 64-bit FPU has FMA */
+#define _HAVE_FAST_FMA 1
+
+#define _fclass_d(_x) (__extension__(                                   \
+                               {                                        \
+                                       long __fclass;                   \
+                                       __asm__("fclass.d %0, %1" :      \
+                                               "=r" (__fclass) :        \
+                                               "f" (_x));               \
+                                       __fclass;                        \
+                               }))
+
 #endif
 
 #if __riscv_flen >= 32
-__declare_riscv_macro_fclass(long)
-_fclass_f(float x)
-{
-	long fclass;
-	__asm __volatile ("fclass.s\t%0, %1" : "=r" (fclass) : "f" (x));
-	return fclass;
-}
+
+/* anything with a 32-bit FPU has FMAF */
+#define _HAVE_FAST_FMAF 1
+
+#define _fclass_f(_x) (__extension__(                                   \
+                               {                                        \
+                                       long __fclass;                   \
+                                       __asm__("fclass.s %0, %1" :      \
+                                               "=r" (__fclass) :        \
+                                               "f" (_x));               \
+                                       __fclass;                        \
+                               }))
+
 #endif
 
-#undef declare_riscv_macro_fclass
 
-#ifdef __declare_riscv_macro
+/**
+ * Not available for all compilers.
+ * In case of absence, fall back to normal function calls
+ */
+#if defined(__GNUC_GNU_INLINE__) || defined(__GNUC_STDC_INLINE__)
+
+# define __declare_riscv_macro(type) extern __inline type __attribute((gnu_inline, always_inline))
 
 #if __riscv_flen >= 64
 
@@ -121,7 +123,10 @@ __declare_riscv_macro(double)
 fmax (double x, double y)
 {
 	double result;
-	__asm__("fmax.d\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
+        if (issignaling(x) || issignaling(y))
+            return x + y;
+
+	__asm__ volatile("fmax.d\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
 	return result;
 }
 
@@ -129,15 +134,24 @@ __declare_riscv_macro(double)
 fmin (double x, double y)
 {
 	double result;
-	__asm__("fmin.d\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
+        if (issignaling(x) || issignaling(y))
+            return x + y;
+
+	__asm__ volatile("fmin.d\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
 	return result;
 }
 
+__declare_riscv_macro(int)
+__finite(double x)
+{
+	long fclass = _fclass_d (x);
+	return (fclass & (FCLASS_INF|FCLASS_NAN)) == 0;
+}
+
 __declare_riscv_macro(int)
 finite(double x)
 {
-	long fclass = _fclass_d (x);
-	return (fclass & (FCLASS_INF|FCLASS_NAN)) == 0;
+        return __finite(x);
 }
 
 __declare_riscv_macro(int)
@@ -157,45 +171,25 @@ __fpclassifyd (double x)
     return FP_NAN;
 }
 
-#ifndef isinf
-__declare_riscv_macro(int)
-isinf (double x)
-{
-	long fclass = _fclass_d (x);
-	return (fclass & FCLASS_INF);
-}
-#endif
-
-#ifndef isnan
-__declare_riscv_macro(int)
-isnan (double x)
-{
-	long fclass = _fclass_d (x);
-	return (fclass & FCLASS_NAN);
-}
-#endif
-
 __declare_riscv_macro(double)
 sqrt (double x)
 {
 	double result;
 #ifdef _WANT_MATH_ERRNO
-        if (x < 0)
+        if (isless(x, 0.0))
             errno = EDOM;
 #endif
-	__asm__("fsqrt.d %0, %1" : "=f" (result) : "f" (x));
+	__asm__ volatile("fsqrt.d %0, %1" : "=f" (result) : "f" (x));
 	return result;
 }
 
-#if HAVE_FAST_FMA
 __declare_riscv_macro(double)
 fma (double x, double y, double z)
 {
 	double result;
-	__asm__("fmadd.d %0, %1, %2, %3" : "=f" (result) : "f" (x), "f" (y), "f" (z));
+	__asm__ volatile("fmadd.d %0, %1, %2, %3" : "=f" (result) : "f" (x), "f" (y), "f" (z));
 	return result;
 }
-#endif
 
 #endif /* __riscv_flen >= 64 */
 
@@ -222,7 +216,10 @@ __declare_riscv_macro(float)
 fmaxf (float x, float y)
 {
 	float result;
-	__asm__("fmax.s\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
+        if (issignaling(x) || issignaling(y))
+            return x + y;
+
+	__asm__ volatile("fmax.s\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
 	return result;
 }
 
@@ -230,15 +227,24 @@ __declare_riscv_macro(float)
 fminf (float x, float y)
 {
 	float result;
-	__asm__("fmin.s\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
+        if (issignaling(x) || issignaling(y))
+            return x + y;
+
+	__asm__ volatile("fmin.s\t%0, %1, %2" : "=f" (result) : "f" (x), "f" (y));
 	return result;
 }
 
+__declare_riscv_macro(int)
+__finitef(float x)
+{
+	long fclass = _fclass_f (x);
+	return (fclass & (FCLASS_INF|FCLASS_NAN)) == 0;
+}
+
 __declare_riscv_macro(int)
 finitef(float x)
 {
-	long fclass = _fclass_f (x);
-	return (fclass & (FCLASS_INF|FCLASS_NAN)) == 0;
+        return __finitef(x);
 }
 
 __declare_riscv_macro(int)
@@ -258,47 +264,30 @@ __fpclassifyf (float x)
     return FP_NAN;
 }
 
-__declare_riscv_macro(int)
-isinff (float x)
-{
-	long fclass = _fclass_f (x);
-	return (fclass & FCLASS_INF);
-}
-
-__declare_riscv_macro(int)
-isnanf (float x)
-{
-	long fclass = _fclass_f (x);
-	return (fclass & FCLASS_NAN);
-}
-
 __declare_riscv_macro(float)
 sqrtf (float x)
 {
 	float result;
 #ifdef _WANT_MATH_ERRNO
-        if (x < 0)
+        if (isless(x, 0.0f))
             errno = EDOM;
 #endif
-	__asm__("fsqrt.s %0, %1" : "=f" (result) : "f" (x));
+	__asm__ volatile("fsqrt.s %0, %1" : "=f" (result) : "f" (x));
 	return result;
 }
 
-
-#if defined(HAVE_FAST_FMAF)
 __declare_riscv_macro(float)
 fmaf (float x, float y, float z)
 {
 	float result;
-	__asm__("fmadd.s %0, %1, %2, %3" : "=f" (result) : "f" (x), "f" (y), "f" (z));
+	__asm__ volatile("fmadd.s %0, %1, %2, %3" : "=f" (result) : "f" (x), "f" (y), "f" (z));
 	return result;
 }
-#endif
 
 #endif /* __riscv_flen >= 32 */
 
-#undef declare_riscv_macro
+#endif /* defined(__GNUC_GNU_INLINE__) || defined(__GNUC_STDC_INLINE__) */
 
-#endif /* defined(declare_riscv_macro) */
+#endif /* __riscv_flen */
 
 #endif /* _MACHINE_MATH_H_ */
