@@ -28,26 +28,8 @@
  * vsscanf
  */
 
-#ifndef _REENT_ONLY
-
 int
-vsscanf (const char *__restrict str,
-       const char *__restrict fmt,
-       va_list ap)
-{
-  return _vsscanf_r (_REENT, str, fmt, ap);
-}
-
-#ifdef _NANO_FORMATTED_IO
-int
-vsiscanf (const char *, const char *, __VALIST)
-       _ATTRIBUTE ((__alias__("vsscanf")));
-#endif
-
-#endif /* !_REENT_ONLY */
-
-int
-_vsscanf_r (struct _reent *ptr,
+vsscanf (
        const char *__restrict str,
        const char *__restrict fmt,
        va_list ap)
@@ -61,11 +43,11 @@ _vsscanf_r (struct _reent *ptr,
   f._ub._base = NULL;
   f._lb._base = NULL;
   f._file = -1;  /* No file. */
-  return __ssvfscanf_r (ptr, &f, fmt, ap);
+  return _ssvfscanf ( &f, fmt, ap);
 }
 
 #ifdef _NANO_FORMATTED_IO
-int
-_vsiscanf_r (struct _reent *, const char *, const char *, __VALIST)
-       _ATTRIBUTE ((__alias__("_vsscanf_r")));
+int __nonnull((1)) _NOTHROW
+vsiscanf (const char *, const char *, __VALIST)
+       _ATTRIBUTE ((__alias__("vsscanf")));
 #endif
