@@ -43,7 +43,7 @@ fgetc(FILE *stream)
 		return EOF;
 
 	if ((unget = __atomic_exchange_ungetc(&stream->unget, 0)) != 0)
-		return (unsigned char) unget;
+                return (unsigned char) (unget - 1);
 
 	rv = stream->get(stream);
 	if (rv < 0) {
@@ -55,7 +55,7 @@ fgetc(FILE *stream)
 	return (unsigned char)rv;
 }
 
-#ifdef HAVE_ALIAS_ATTRIBUTE
+#ifdef _HAVE_ALIAS_ATTRIBUTE
 __strong_reference(fgetc, getc);
 #elif !defined(getc)
 int getc(FILE *stream) { return fgetc(stream); }
